package com.eitc.patient.utils;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eitc.common.core.domain.entity.SysRole;
import com.eitc.common.core.domain.entity.SysUser;
import com.eitc.common.core.domain.model.LoginUser;
import com.eitc.common.core.text.Convert;
import com.eitc.common.utils.PatientBaseUtil;
import com.eitc.common.utils.SecurityUtils;
import com.eitc.common.utils.spring.SpringUtils;
import com.eitc.framework.security.context.PermissionContextHolder;
import com.eitc.patient.domain.ClinicInfo;
import com.eitc.patient.domain.SchedulingDetails;
import com.eitc.patient.domain.SchedulingStatus;
import com.eitc.patient.service.IClinicInfoService;
import com.eitc.patient.service.ISchedulingDetailsService;
import com.eitc.patient.service.ISchedulingStatusService;
import com.eitc.system.service.ISysUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * PatientUtil
 *
 * @author mtx
 * @date: 2024/06/18 17:32
 */
@Component
public class PatientUtil {
    private final static ISchedulingDetailsService schedulingDetailsService = SpringUtils.getBean(ISchedulingDetailsService.class);
    private final static ISchedulingStatusService schedulingStatusService = SpringUtils.getBean(ISchedulingStatusService.class);
    private final static IClinicInfoService clinicInfoService = SpringUtils.getBean(IClinicInfoService.class);

    /**
     * 根据给定的星期几的数字返回对应的中文星期名称。
     * 使用Calendar类中的常量来表示星期几，提高代码的可读性和维护性。
     *
     * @param week 根据Calendar类中定义的常量，表示星期几的整数。
     *             MONDAY表示星期一，TUESDAY表示星期二，以此类推，SUNDAY表示星期日。
     * @return 对应于输入参数的中文星期名称。如果输入值不是有效的星期数，则返回空字符串。
     */
    public static String getWeekDay(int week) {
        switch (week) {
            case Calendar.MONDAY:
                return "星期一";
            case Calendar.TUESDAY:
                return "星期二";
            case Calendar.WEDNESDAY:
                return "星期三";
            case Calendar.THURSDAY:
                return "星期四";
            case Calendar.FRIDAY:
                return "星期五";
            case Calendar.SATURDAY:
                return "星期六";
            case Calendar.SUNDAY:
                return "星期日";
            default:
                return "";
        }
    }


    /**
     * 检查医生当前工作状态是否处于工作时间。
     *
     * @param userId 医生用户ID
     * @return 当前是否为工作状态
     */
    public static boolean checkDoctorWorkStatus(String userId, Date selectDate) {
        boolean flag = false;
        // 创建Calendar实例，用于后续时间处理
        Calendar instance = Calendar.getInstance();
        if (Objects.nonNull(selectDate)) {
            instance.setTime(selectDate);
        } else {
            selectDate = new Date();
            instance.setTime(selectDate);
        }
        // 移除毫秒部分，确保时间比较的准确性
        instance.set(Calendar.MILLISECOND, 0);

        // 查询当前医生当天的工作安排详情
        List<SchedulingDetails> list = schedulingDetailsService.list(Wrappers.<SchedulingDetails>lambdaQuery()
                .eq(SchedulingDetails::getDoctorId, userId)
                .eq(SchedulingDetails::getWorkYear, instance.get(Calendar.YEAR))
                .eq(SchedulingDetails::getWorkMonth, instance.get(Calendar.MONTH) + 1)
                .eq(SchedulingDetails::getWorkDay, instance.get(Calendar.DAY_OF_MONTH)));

        // 如果有工作安排，则进一步判断当前时间是否在工作时间内
        if (CollectionUtils.isNotEmpty(list)) {
            SchedulingDetails details = list.get(0);
            // 获取工作状态ID
            String workStatusId = details.getWorkStatus();
            // 根据工作状态ID查询工作状态详情
            SchedulingStatus schedulingStatus = schedulingStatusService.getById(workStatusId);
            Date startTime = null;
            Date startDate = null;
            Date endTime = null;
            Date endDate = null;

            if (schedulingStatus != null) {
                // 获取工作开始时间，并将其设置到Calendar对象中
                startTime = schedulingStatus.getStartTime();
                if (startTime != null) {
                    instance.set(Calendar.HOUR_OF_DAY, Integer.parseInt(String.format("%tH", startTime)));
                    instance.set(Calendar.MINUTE, Integer.parseInt(String.format("%tM", startTime)));
                    instance.set(Calendar.SECOND, Integer.parseInt(String.format("%tS", startTime)));
                }
                startDate = instance.getTime();

                // 获取工作结束时间，并将其设置到Calendar对象中
                endTime = schedulingStatus.getEndTime();
                if (endTime != null) {
                    instance.set(Calendar.HOUR_OF_DAY, Integer.parseInt(String.format("%tH", endTime)));
                    instance.set(Calendar.MINUTE, Integer.parseInt(String.format("%tM", endTime)));
                    instance.set(Calendar.SECOND, Integer.parseInt(String.format("%tS", endTime)));
                }
                endDate = instance.getTime();
            }

            // 比较当前时间是否在工作时间范围内
            if (startTime != null && endTime != null && selectDate.compareTo(startDate) >= 0 && selectDate.compareTo(endDate) <= 0) {
                flag = true;
            }
        }
        return flag;
    }


    /**
     * 生成医疗记录号。
     * 本方法通过结合当前日期和时间以及Redis的自增功能来生成唯一的医疗记录号。
     * 记录号的格式为：缩写 + 当前日期时间 + 三位自增数字。
     * 使用Redis的自增功能确保了记录号的唯一性。
     *
     * @return String 生成的医疗记录号。
     */
    public static String findMedicalRecordNum() {
        List<ClinicInfo> list = clinicInfoService.list();
        if (CollectionUtils.isEmpty(list)) {
            throw new RuntimeException("未找到任何诊所信息");
        }
        String clinicCode = list.get(0).getClinicCode();
        if (StringUtils.isBlank(clinicCode)) {
            throw new RuntimeException("未找到诊所编号");
        }

        return clinicCode + PatientBaseUtil.generateRandom(12);
    }


    /**
     * 根据当前登录用户查询用户列表
     * 该方法主要用于获取经过权限过滤的用户列表，确保普通用户只能访问他们有权限查看的用户信息
     * 如果当前用户是超级管理员，则忽略权限设置，返回所有用户
     *
     * @return 过滤后的用户列表，如果当前用户未登录或无权限，返回空列表
     */
    public static List<SysUser> findUserListByLoginUser() {
        // 获取当前的用户
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (com.eitc.common.utils.StringUtils.isNotNull(loginUser)) {
            SysUser currentUser = loginUser.getUser();
            // 如果是超级管理员，则不过滤数据
            if (com.eitc.common.utils.StringUtils.isNotNull(currentUser) && !currentUser.isAdmin()) {
                String permission = PermissionContextHolder.getContext();
                return findUserListByPermission(currentUser, permission);
            }
        }
        return new ArrayList<>();
    }

    /**
     * 全部数据权限
     */
    public static final String DATA_SCOPE_ALL = "1";

    /**
     * 自定数据权限
     */
    public static final String DATA_SCOPE_CUSTOM = "2";

    /**
     * 部门数据权限
     */
    public static final String DATA_SCOPE_DEPT = "3";

    /**
     * 部门及以下数据权限
     */
    public static final String DATA_SCOPE_DEPT_AND_CHILD = "4";

    /**
     * 仅本人数据权限
     */
    public static final String DATA_SCOPE_SELF = "5";

    private static List<SysUser> findUserListByPermission(SysUser user, String permission) {
        ISysUserService sysUserService = SpringUtils.getBean(ISysUserService.class);
        List<SysUser> userList = new ArrayList<>();
        List<String> conditions = new ArrayList<>();

        for (SysRole role : user.getRoles()) {
            String dataScope = role.getDataScope();
            if (!DATA_SCOPE_CUSTOM.equals(dataScope) && conditions.contains(dataScope)) {
                continue;
            }
            if (com.eitc.common.utils.StringUtils.isNotEmpty(permission)
                    && com.eitc.common.utils.StringUtils.isNotEmpty(role.getPermissions())
                    && !com.eitc.common.utils.StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission))) {
                continue;
            }
            if (DATA_SCOPE_ALL.equals(dataScope)) {
                // 查询全部
                conditions.add(dataScope);
                userList = new ArrayList<>();
                break;
            } else if (DATA_SCOPE_CUSTOM.equals(dataScope)) {
                userList.addAll(sysUserService.selectUserListByRoleId(role.getRoleId()));
            } else if (DATA_SCOPE_DEPT.equals(dataScope)) {
                userList.addAll(sysUserService.selectUserListByDeptId(user.getDeptId()));
            } else if (DATA_SCOPE_DEPT_AND_CHILD.equals(dataScope)) {
                userList.addAll(sysUserService.selectUserListByDeptIdAncestors(user.getDeptId()));
            } else if (DATA_SCOPE_SELF.equals(dataScope)) {
                LoginUser loginUser = SecurityUtils.getLoginUser();
                if (com.eitc.common.utils.StringUtils.isNotNull(loginUser)) {
                    userList.add(loginUser.getUser());
                } else {
                    userList.add(new SysUser());
                }
            }
            conditions.add(dataScope);
        }

        // 多角色情况下，所有角色都不包含传递过来的权限字符，这个时候sqlString也会为空，所以要限制一下,不查询任何数据
        if (com.eitc.common.utils.StringUtils.isEmpty(conditions)) {
            // 不查询任何数据;
            userList.addAll(sysUserService.selectUserListByDeptId(0L));
        }
        return userList.stream().distinct().collect(Collectors.toList());
    }
}
